# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meshtrade/compliance/client/v1/identification_document_type.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'meshtrade/compliance/client/v1/identification_document_type.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAmeshtrade/compliance/client/v1/identification_document_type.proto\x12\x1emeshtrade.compliance.client.v1*\x88\x02\n\x1aIdentificationDocumentType\x12,\n(IDENTIFICATION_DOCUMENT_TYPE_UNSPECIFIED\x10\x00\x12)\n%IDENTIFICATION_DOCUMENT_TYPE_PASSPORT\x10\x01\x12,\n(IDENTIFICATION_DOCUMENT_TYPE_NATIONAL_ID\x10\x02\x12\x30\n,IDENTIFICATION_DOCUMENT_TYPE_DRIVERS_LICENSE\x10\x03\x12\x31\n-IDENTIFICATION_DOCUMENT_TYPE_RESIDENCE_PERMIT\x10\x04\x42\x63\n%co.meshtrade.api.compliance.client.v1Z:github.com/meshtrade/api/go/compliance/client/v1;client_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtrade.compliance.client.v1.identification_document_type_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n%co.meshtrade.api.compliance.client.v1Z:github.com/meshtrade/api/go/compliance/client/v1;client_v1'
  _globals['_IDENTIFICATIONDOCUMENTTYPE']._serialized_start=102
  _globals['_IDENTIFICATIONDOCUMENTTYPE']._serialized_end=366
# @@protoc_insertion_point(module_scope)



from buf.validate import validate_pb2 as _validate_pb2
from meshtrade.option.method_options.v1 import method_options_pb2 as _method_options_pb2
from meshtrade.testing.ledger.token_tap.v1 import option_pb2 as _option_pb2
from meshtrade.type.v1 import amount_pb2 as _amount_pb2
from meshtrade.type.v1 import token_pb2 as _token_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class InitialiseTokenTapsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class InitialiseTokenTapsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ListTokenTapsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ListTokenTapsResponse(_message.Message):
    __slots__ = ("tokens",)
    TOKENS_FIELD_NUMBER: _ClassVar[int]
    tokens: _containers.RepeatedCompositeFieldContainer[_token_pb2.Token]
    def __init__(self, tokens: _Optional[_Iterable[_Union[_token_pb2.Token, _Mapping]]] = ...) -> None: ...

class MintTokenRequest(_message.Message):
    __slots__ = ("amount", "address", "options")
    AMOUNT_FIELD_NUMBER: _ClassVar[int]
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    amount: _amount_pb2.Amount
    address: str
    options: _option_pb2.MintTokenOptions
    def __init__(self, amount: _Optional[_Union[_amount_pb2.Amount, _Mapping]] = ..., address: _Optional[str] = ..., options: _Optional[_Union[_option_pb2.MintTokenOptions, _Mapping]] = ...) -> None: ...

class MintTokenResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meshtrade/reporting/account_report/v1/fee_entry.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'meshtrade/reporting/account_report/v1/fee_entry.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from meshtrade.type.v1 import amount_pb2 as meshtrade_dot_type_dot_v1_dot_amount__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5meshtrade/reporting/account_report/v1/fee_entry.proto\x12%meshtrade.reporting.account_report.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1emeshtrade/type/v1/amount.proto\"\xc7\x02\n\x08\x46\x65\x65\x45ntry\x12\x45\n\x10transaction_date\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0ftransactionDate\x12%\n\x0etransaction_id\x18\x02 \x01(\tR\rtransactionId\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12\x31\n\x06\x61mount\x18\x04 \x01(\x0b\x32\x19.meshtrade.type.v1.AmountR\x06\x61mount\x12Q\n\x17reported_currency_value\x18\x05 \x01(\x0b\x32\x19.meshtrade.type.v1.AmountR\x15reportedCurrencyValue\x12%\n\x0etoken_currency\x18\x06 \x01(\tR\rtokenCurrencyBy\n,co.meshtrade.api.reporting.account_report.v1ZIgithub.com/meshtrade/api/go/reporting/account_report/v1;account_report_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtrade.reporting.account_report.v1.fee_entry_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n,co.meshtrade.api.reporting.account_report.v1ZIgithub.com/meshtrade/api/go/reporting/account_report/v1;account_report_v1'
  _globals['_FEEENTRY']._serialized_start=162
  _globals['_FEEENTRY']._serialized_end=489
# @@protoc_insertion_point(module_scope)



"""Limit Order v1 package."""

# ===================================================================
# AUTO-GENERATED SECTION - ONLY EDIT BELOW THE CLOSING COMMENT BLOCK
# ===================================================================
# This section is automatically managed by protoc-gen-meshpy.
#
# DO NOT EDIT ANYTHING IN THIS SECTION MANUALLY!
# Your changes will be overwritten during code generation.
#
# To add custom imports and exports, scroll down to the
# "MANUAL SECTION" indicated below.
# ===================================================================

# Generated protobuf imports
from .limit_order_pb2 import LimitOrder, LimitOrderSide, LimitOrderState
from .service_pb2 import (
    CancelLimitOrderRequest,
    CreateLimitOrderRequest,
    GetLimitOrderByExternalReferenceRequest,
    GetLimitOrderRequest,
    ListLimitOrdersRequest,
    ListLimitOrdersResponse,
    MonitorLimitOrderRequest,
    SearchLimitOrdersRequest,
    SearchLimitOrdersResponse,
)

# Generated service imports
from .service_meshpy import (
    LimitOrderService,
    LimitOrderServiceGRPCClient,
    LimitOrderServiceGRPCClientInterface,
)

# ===================================================================
# END OF AUTO-GENERATED SECTION
# ===================================================================
#
# MANUAL SECTION - ADD YOUR CUSTOM IMPORTS AND EXPORTS BELOW
#
# You can safely add your own imports, functions, classes, and exports
# in this section. They will be preserved across code generation.
#
# Example:
#   from my_custom_module import my_function
#
# ===================================================================

# ===================================================================
# MODULE EXPORTS
# ===================================================================
# Combined auto-generated and manual exports
__all__ = [
    # Generated exports
    "CancelLimitOrderRequest",
    "CreateLimitOrderRequest",
    "GetLimitOrderByExternalReferenceRequest",
    "GetLimitOrderRequest",
    "LimitOrder",
    "LimitOrderService",
    "LimitOrderServiceGRPCClient",
    "LimitOrderServiceGRPCClientInterface",
    "LimitOrderSide",
    "LimitOrderState",
    "ListLimitOrdersRequest",
    "ListLimitOrdersResponse",
    "MonitorLimitOrderRequest",
    "SearchLimitOrdersRequest",
    "SearchLimitOrdersResponse",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meshtrade/market_data/price/v1/service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'meshtrade/market_data/price/v1/service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from meshtrade.market_data.price.v1 import price_pb2 as meshtrade_dot_market__data_dot_price_dot_v1_dot_price__pb2
from meshtrade.option.method_options.v1 import method_options_pb2 as meshtrade_dot_option_dot_method__options_dot_v1_dot_method__options__pb2
from meshtrade.type.v1 import token_pb2 as meshtrade_dot_type_dot_v1_dot_token__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,meshtrade/market_data/price/v1/service.proto\x12\x1emeshtrade.market_data.price.v1\x1a\x1b\x62uf/validate/validate.proto\x1a*meshtrade/market_data/price/v1/price.proto\x1a\x37meshtrade/option/method_options/v1/method_options.proto\x1a\x1dmeshtrade/type/v1/token.proto\"\xa7\x01\n!GetCurrentPriceByTokenPairRequest\x12?\n\nbase_token\x18\x01 \x01(\x0b\x32\x18.meshtrade.type.v1.TokenB\x06\xbaH\x03\xc8\x01\x01R\tbaseToken\x12\x41\n\x0bquote_token\x18\x02 \x01(\x0b\x32\x18.meshtrade.type.v1.TokenB\x06\xbaH\x03\xc8\x01\x01R\nquoteToken2\xb3\x01\n\x0cPriceService\x12\xa2\x01\n\x1aGetCurrentPriceByTokenPair\x12\x41.meshtrade.market_data.price.v1.GetCurrentPriceByTokenPairRequest\x1a%.meshtrade.market_data.price.v1.Price\"\x1a\xb2\xb5\x18\x16\x08\x01\x10\x02\x1a\x10\x80\xa4\xe8\x03\x81\xa4\xe8\x03\x82\xa4\xe8\x03\x83\xa4\xe8\x03\x42\x62\n%co.meshtrade.api.market_data.price.v1Z9github.com/meshtrade/api/go/market_data/price/v1;price_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtrade.market_data.price.v1.service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n%co.meshtrade.api.market_data.price.v1Z9github.com/meshtrade/api/go/market_data/price/v1;price_v1'
  _globals['_GETCURRENTPRICEBYTOKENPAIRREQUEST'].fields_by_name['base_token']._loaded_options = None
  _globals['_GETCURRENTPRICEBYTOKENPAIRREQUEST'].fields_by_name['base_token']._serialized_options = b'\272H\003\310\001\001'
  _globals['_GETCURRENTPRICEBYTOKENPAIRREQUEST'].fields_by_name['quote_token']._loaded_options = None
  _globals['_GETCURRENTPRICEBYTOKENPAIRREQUEST'].fields_by_name['quote_token']._serialized_options = b'\272H\003\310\001\001'
  _globals['_PRICESERVICE'].methods_by_name['GetCurrentPriceByTokenPair']._loaded_options = None
  _globals['_PRICESERVICE'].methods_by_name['GetCurrentPriceByTokenPair']._serialized_options = b'\262\265\030\026\010\001\020\002\032\020\200\244\350\003\201\244\350\003\202\244\350\003\203\244\350\003'
  _globals['_GETCURRENTPRICEBYTOKENPAIRREQUEST']._serialized_start=242
  _globals['_GETCURRENTPRICEBYTOKENPAIRREQUEST']._serialized_end=409
  _globals['_PRICESERVICE']._serialized_start=412
  _globals['_PRICESERVICE']._serialized_end=591
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meshtrade/iam/user/v1/service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'meshtrade/iam/user/v1/service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from meshtrade.iam.user.v1 import user_pb2 as meshtrade_dot_iam_dot_user_dot_v1_dot_user__pb2
from meshtrade.option.method_options.v1 import method_options_pb2 as meshtrade_dot_option_dot_method__options_dot_v1_dot_method__options__pb2
from meshtrade.type.v1 import sorting_pb2 as meshtrade_dot_type_dot_v1_dot_sorting__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#meshtrade/iam/user/v1/service.proto\x12\x15meshtrade.iam.user.v1\x1a\x1b\x62uf/validate/validate.proto\x1a meshtrade/iam/user/v1/user.proto\x1a\x37meshtrade/option/method_options/v1/method_options.proto\x1a\x1fmeshtrade/type/v1/sorting.proto\"\xdb\x01\n\x18\x41ssignRolesToUserRequest\x12O\n\x04name\x18\x01 \x01(\tB;\xbaH8r32.^users/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}$\x98\x01 \xc8\x01\x01R\x04name\x12n\n\x05roles\x18\x02 \x03(\tBX\xbaHU\x92\x01O\"MrK\x10/\x18\x30\x32\x45^groups/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}/roles/[1-9][0-9]{6,7}$\xc8\x01\x01R\x05roles\"\xdd\x01\n\x1aRevokeRolesFromUserRequest\x12O\n\x04name\x18\x01 \x01(\tB;\xbaH8r32.^users/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}$\x98\x01 \xc8\x01\x01R\x04name\x12n\n\x05roles\x18\x02 \x03(\tBX\xbaHU\x92\x01O\"MrK\x10/\x18\x30\x32\x45^groups/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}/roles/[1-9][0-9]{6,7}$\xc8\x01\x01R\x05roles\"a\n\x0eGetUserRequest\x12O\n\x04name\x18\x01 \x01(\tB;\xbaH8r32.^users/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}$\x98\x01 \xc8\x01\x01R\x04name\"9\n\x15GetUserByEmailRequest\x12 \n\x05\x65mail\x18\x01 \x01(\tB\n\xbaH\x07r\x02`\x01\xc8\x01\x01R\x05\x65mail\"\x94\x02\n\x10ListUsersRequest\x12I\n\x07sorting\x18\x01 \x01(\x0b\x32/.meshtrade.iam.user.v1.ListUsersRequest.SortingR\x07sorting\x1a\xb4\x01\n\x07Sorting\x12r\n\x05\x66ield\x18\x01 \x01(\tB\\\xbaHYr\tR\x00R\x05\x65mail\xba\x01K\n\x0b\x66ield.valid\x12%field must be one of: email, or empty\x1a\x15this in [\'\', \'email\']R\x05\x66ield\x12\x35\n\x05order\x18\x02 \x01(\x0e\x32\x1f.meshtrade.type.v1.SortingOrderR\x05order\"F\n\x11ListUsersResponse\x12\x31\n\x05users\x18\x01 \x03(\x0b\x32\x1b.meshtrade.iam.user.v1.UserR\x05users\"3\n\x12SearchUsersRequest\x12\x1d\n\x05\x65mail\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x18\x64R\x05\x65mail\"H\n\x13SearchUsersResponse\x12\x31\n\x05users\x18\x01 \x03(\x0b\x32\x1b.meshtrade.iam.user.v1.UserR\x05users\"L\n\x11\x43reateUserRequest\x12\x37\n\x04user\x18\x01 \x01(\x0b\x32\x1b.meshtrade.iam.user.v1.UserB\x06\xbaH\x03\xc8\x01\x01R\x04user\"L\n\x11UpdateUserRequest\x12\x37\n\x04user\x18\x01 \x01(\x0b\x32\x1b.meshtrade.iam.user.v1.UserB\x06\xbaH\x03\xc8\x01\x01R\x04user2\xb4\x07\n\x0bUserService\x12u\n\x11\x41ssignRolesToUser\x12/.meshtrade.iam.user.v1.AssignRolesToUserRequest\x1a\x1b.meshtrade.iam.user.v1.User\"\x12\xb2\xb5\x18\x0e\x08\x02\x10\x02\x1a\x08\xc0\x8d\xb7\x01\xc6\x8d\xb7\x01\x12y\n\x13RevokeRolesFromUser\x12\x31.meshtrade.iam.user.v1.RevokeRolesFromUserRequest\x1a\x1b.meshtrade.iam.user.v1.User\"\x12\xb2\xb5\x18\x0e\x08\x02\x10\x02\x1a\x08\xc0\x8d\xb7\x01\xc6\x8d\xb7\x01\x12i\n\x07GetUser\x12%.meshtrade.iam.user.v1.GetUserRequest\x1a\x1b.meshtrade.iam.user.v1.User\"\x1a\xb2\xb5\x18\x16\x08\x01\x10\x02\x1a\x10\xc0\x8d\xb7\x01\xc1\x8d\xb7\x01\xc6\x8d\xb7\x01\xc7\x8d\xb7\x01\x12w\n\x0eGetUserByEmail\x12,.meshtrade.iam.user.v1.GetUserByEmailRequest\x1a\x1b.meshtrade.iam.user.v1.User\"\x1a\xb2\xb5\x18\x16\x08\x01\x10\x02\x1a\x10\xc0\x8d\xb7\x01\xc1\x8d\xb7\x01\xc6\x8d\xb7\x01\xc7\x8d\xb7\x01\x12z\n\tListUsers\x12\'.meshtrade.iam.user.v1.ListUsersRequest\x1a(.meshtrade.iam.user.v1.ListUsersResponse\"\x1a\xb2\xb5\x18\x16\x08\x01\x10\x02\x1a\x10\xc0\x8d\xb7\x01\xc1\x8d\xb7\x01\xc6\x8d\xb7\x01\xc7\x8d\xb7\x01\x12\x80\x01\n\x0bSearchUsers\x12).meshtrade.iam.user.v1.SearchUsersRequest\x1a*.meshtrade.iam.user.v1.SearchUsersResponse\"\x1a\xb2\xb5\x18\x16\x08\x01\x10\x02\x1a\x10\xc0\x8d\xb7\x01\xc1\x8d\xb7\x01\xc6\x8d\xb7\x01\xc7\x8d\xb7\x01\x12g\n\nCreateUser\x12(.meshtrade.iam.user.v1.CreateUserRequest\x1a\x1b.meshtrade.iam.user.v1.User\"\x12\xb2\xb5\x18\x0e\x08\x02\x10\x02\x1a\x08\xc0\x8d\xb7\x01\xc6\x8d\xb7\x01\x12g\n\nUpdateUser\x12(.meshtrade.iam.user.v1.UpdateUserRequest\x1a\x1b.meshtrade.iam.user.v1.User\"\x12\xb2\xb5\x18\x0e\x08\x02\x10\x02\x1a\x08\xc0\x8d\xb7\x01\xc6\x8d\xb7\x01\x42O\n\x1c\x63o.meshtrade.api.iam.user.v1Z/github.com/meshtrade/api/go/iam/user/v1;user_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtrade.iam.user.v1.service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034co.meshtrade.api.iam.user.v1Z/github.com/meshtrade/api/go/iam/user/v1;user_v1'
  _globals['_ASSIGNROLESTOUSERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_ASSIGNROLESTOUSERREQUEST'].fields_by_name['name']._serialized_options = b'\272H8r32.^users/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}$\230\001 \310\001\001'
  _globals['_ASSIGNROLESTOUSERREQUEST'].fields_by_name['roles']._loaded_options = None
  _globals['_ASSIGNROLESTOUSERREQUEST'].fields_by_name['roles']._serialized_options = b'\272HU\222\001O\"MrK\020/\03002E^groups/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}/roles/[1-9][0-9]{6,7}$\310\001\001'
  _globals['_REVOKEROLESFROMUSERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_REVOKEROLESFROMUSERREQUEST'].fields_by_name['name']._serialized_options = b'\272H8r32.^users/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}$\230\001 \310\001\001'
  _globals['_REVOKEROLESFROMUSERREQUEST'].fields_by_name['roles']._loaded_options = None
  _globals['_REVOKEROLESFROMUSERREQUEST'].fields_by_name['roles']._serialized_options = b'\272HU\222\001O\"MrK\020/\03002E^groups/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}/roles/[1-9][0-9]{6,7}$\310\001\001'
  _globals['_GETUSERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETUSERREQUEST'].fields_by_name['name']._serialized_options = b'\272H8r32.^users/[0123456789ABCDEFGHJKMNPQRSTVWXYZ]{26}$\230\001 \310\001\001'
  _globals['_GETUSERBYEMAILREQUEST'].fields_by_name['email']._loaded_options = None
  _globals['_GETUSERBYEMAILREQUEST'].fields_by_name['email']._serialized_options = b'\272H\007r\002`\001\310\001\001'
  _globals['_LISTUSERSREQUEST_SORTING'].fields_by_name['field']._loaded_options = None
  _globals['_LISTUSERSREQUEST_SORTING'].fields_by_name['field']._serialized_options = b'\272HYr\tR\000R\005email\272\001K\n\013field.valid\022%field must be one of: email, or empty\032\025this in [\'\', \'email\']'
  _globals['_SEARCHUSERSREQUEST'].fields_by_name['email']._loaded_options = None
  _globals['_SEARCHUSERSREQUEST'].fields_by_name['email']._serialized_options = b'\272H\004r\002\030d'
  _globals['_CREATEUSERREQUEST'].fields_by_name['user']._loaded_options = None
  _globals['_CREATEUSERREQUEST'].fields_by_name['user']._serialized_options = b'\272H\003\310\001\001'
  _globals['_UPDATEUSERREQUEST'].fields_by_name['user']._loaded_options = None
  _globals['_UPDATEUSERREQUEST'].fields_by_name['user']._serialized_options = b'\272H\003\310\001\001'
  _globals['_USERSERVICE'].methods_by_name['AssignRolesToUser']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['AssignRolesToUser']._serialized_options = b'\262\265\030\016\010\002\020\002\032\010\300\215\267\001\306\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['RevokeRolesFromUser']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['RevokeRolesFromUser']._serialized_options = b'\262\265\030\016\010\002\020\002\032\010\300\215\267\001\306\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['GetUser']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['GetUser']._serialized_options = b'\262\265\030\026\010\001\020\002\032\020\300\215\267\001\301\215\267\001\306\215\267\001\307\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['GetUserByEmail']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['GetUserByEmail']._serialized_options = b'\262\265\030\026\010\001\020\002\032\020\300\215\267\001\301\215\267\001\306\215\267\001\307\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['ListUsers']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['ListUsers']._serialized_options = b'\262\265\030\026\010\001\020\002\032\020\300\215\267\001\301\215\267\001\306\215\267\001\307\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['SearchUsers']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['SearchUsers']._serialized_options = b'\262\265\030\026\010\001\020\002\032\020\300\215\267\001\301\215\267\001\306\215\267\001\307\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['CreateUser']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['CreateUser']._serialized_options = b'\262\265\030\016\010\002\020\002\032\010\300\215\267\001\306\215\267\001'
  _globals['_USERSERVICE'].methods_by_name['UpdateUser']._loaded_options = None
  _globals['_USERSERVICE'].methods_by_name['UpdateUser']._serialized_options = b'\262\265\030\016\010\002\020\002\032\010\300\215\267\001\306\215\267\001'
  _globals['_ASSIGNROLESTOUSERREQUEST']._serialized_start=216
  _globals['_ASSIGNROLESTOUSERREQUEST']._serialized_end=435
  _globals['_REVOKEROLESFROMUSERREQUEST']._serialized_start=438
  _globals['_REVOKEROLESFROMUSERREQUEST']._serialized_end=659
  _globals['_GETUSERREQUEST']._serialized_start=661
  _globals['_GETUSERREQUEST']._serialized_end=758
  _globals['_GETUSERBYEMAILREQUEST']._serialized_start=760
  _globals['_GETUSERBYEMAILREQUEST']._serialized_end=817
  _globals['_LISTUSERSREQUEST']._serialized_start=820
  _globals['_LISTUSERSREQUEST']._serialized_end=1096
  _globals['_LISTUSERSREQUEST_SORTING']._serialized_start=916
  _globals['_LISTUSERSREQUEST_SORTING']._serialized_end=1096
  _globals['_LISTUSERSRESPONSE']._serialized_start=1098
  _globals['_LISTUSERSRESPONSE']._serialized_end=1168
  _globals['_SEARCHUSERSREQUEST']._serialized_start=1170
  _globals['_SEARCHUSERSREQUEST']._serialized_end=1221
  _globals['_SEARCHUSERSRESPONSE']._serialized_start=1223
  _globals['_SEARCHUSERSRESPONSE']._serialized_end=1295
  _globals['_CREATEUSERREQUEST']._serialized_start=1297
  _globals['_CREATEUSERREQUEST']._serialized_end=1373
  _globals['_UPDATEUSERREQUEST']._serialized_start=1375
  _globals['_UPDATEUSERREQUEST']._serialized_end=1451
  _globals['_USERSERVICE']._serialized_start=1454
  _globals['_USERSERVICE']._serialized_end=2402
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class SourceOfIncomeAndWealth(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SOURCE_OF_INCOME_AND_WEALTH_UNSPECIFIED: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_ALLOWANCES: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_BURSARY: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_COURT_ORDER: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_LOAN_FINANCIAL_INSTITUTION: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_LOAN_OTHER: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_MAINTENANCE: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_MATURING_INVESTMENTS: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_PENSION: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_RENTAL_INCOME: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_COMPANY_PROFITS: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_COMPANY_SALE: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_DECEASED_ESTATE: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_DIVORCE_SETTLEMENT: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_GIFT_OR_DONATION: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_INCOME_FROM_EMPLOYMENT: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_INCOME_FROM_PREVIOUS_EMPLOYMENT: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_INHERITANCE: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_LOTTERY_WINNINGS_OR_GAMBLING: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_SALE_OF_ASSET: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_SALE_OF_SHARES: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_SAVINGS_INVESTMENT_OR_DIVIDEND: _ClassVar[SourceOfIncomeAndWealth]
    SOURCE_OF_INCOME_AND_WEALTH_TRUST_DISTRIBUTIONS: _ClassVar[SourceOfIncomeAndWealth]
SOURCE_OF_INCOME_AND_WEALTH_UNSPECIFIED: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_ALLOWANCES: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_BURSARY: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_COURT_ORDER: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_LOAN_FINANCIAL_INSTITUTION: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_LOAN_OTHER: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_MAINTENANCE: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_MATURING_INVESTMENTS: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_PENSION: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_RENTAL_INCOME: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_COMPANY_PROFITS: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_COMPANY_SALE: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_DECEASED_ESTATE: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_DIVORCE_SETTLEMENT: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_GIFT_OR_DONATION: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_INCOME_FROM_EMPLOYMENT: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_INCOME_FROM_PREVIOUS_EMPLOYMENT: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_INHERITANCE: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_LOTTERY_WINNINGS_OR_GAMBLING: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_SALE_OF_ASSET: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_SALE_OF_SHARES: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_SAVINGS_INVESTMENT_OR_DIVIDEND: SourceOfIncomeAndWealth
SOURCE_OF_INCOME_AND_WEALTH_TRUST_DISTRIBUTIONS: SourceOfIncomeAndWealth



"""Group v1 package."""

# ===================================================================
# AUTO-GENERATED SECTION - ONLY EDIT BELOW THE CLOSING COMMENT BLOCK
# ===================================================================
# This section is automatically managed by protoc-gen-meshpy.
#
# DO NOT EDIT ANYTHING IN THIS SECTION MANUALLY!
# Your changes will be overwritten during code generation.
#
# To add custom imports and exports, scroll down to the
# "MANUAL SECTION" indicated below.
# ===================================================================

# Generated protobuf imports
from .group_pb2 import Group
from .service_pb2 import (
    CreateGroupRequest,
    GetGroupRequest,
    ListGroupsRequest,
    ListGroupsResponse,
    SearchGroupsRequest,
    SearchGroupsResponse,
    UpdateGroupRequest,
)

# Generated service imports
from .service_meshpy import (
    GroupService,
    GroupServiceGRPCClient,
    GroupServiceGRPCClientInterface,
)

# ===================================================================
# END OF AUTO-GENERATED SECTION
# ===================================================================
#
# MANUAL SECTION - ADD YOUR CUSTOM IMPORTS AND EXPORTS BELOW
#
# You can safely add your own imports, functions, classes, and exports
# in this section. They will be preserved across code generation.
#
# Example:
#   from my_custom_module import my_function
#
# ===================================================================

# ===================================================================
# MODULE EXPORTS
# ===================================================================
# Combined auto-generated and manual exports
__all__ = [
    # Generated exports
    "CreateGroupRequest",
    "GetGroupRequest",
    "Group",
    "GroupService",
    "GroupServiceGRPCClient",
    "GroupServiceGRPCClientInterface",
    "ListGroupsRequest",
    "ListGroupsResponse",
    "SearchGroupsRequest",
    "SearchGroupsResponse",
    "UpdateGroupRequest",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meshtrade/market_data/price/v1/price.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'meshtrade/market_data/price/v1/price.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from meshtrade.type.v1 import decimal_pb2 as meshtrade_dot_type_dot_v1_dot_decimal__pb2
from meshtrade.type.v1 import token_pb2 as meshtrade_dot_type_dot_v1_dot_token__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*meshtrade/market_data/price/v1/price.proto\x12\x1emeshtrade.market_data.price.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1fmeshtrade/type/v1/decimal.proto\x1a\x1dmeshtrade/type/v1/token.proto\"\xe4\x01\n\x05Price\x12\x37\n\nbase_token\x18\x01 \x01(\x0b\x32\x18.meshtrade.type.v1.TokenR\tbaseToken\x12\x39\n\x0bquote_token\x18\x02 \x01(\x0b\x32\x18.meshtrade.type.v1.TokenR\nquoteToken\x12\x37\n\tmid_price\x18\x03 \x01(\x0b\x32\x1a.meshtrade.type.v1.DecimalR\x08midPrice\x12.\n\x04time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x04timeBb\n%co.meshtrade.api.market_data.price.v1Z9github.com/meshtrade/api/go/market_data/price/v1;price_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtrade.market_data.price.v1.price_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n%co.meshtrade.api.market_data.price.v1Z9github.com/meshtrade/api/go/market_data/price/v1;price_v1'
  _globals['_PRICE']._serialized_start=176
  _globals['_PRICE']._serialized_end=404
# @@protoc_insertion_point(module_scope)



"""Decimal operations utility functions for Mesh API.

This module provides arithmetic, comparison, and utility functions for working
with protobuf Decimal messages, offering a Pythonic API for decimal arithmetic
with consistent precision handling.
"""

from decimal import ROUND_HALF_UP

from .decimal_built_in_conversions import decimal_to_built_in
from .decimal_pb2 import Decimal


def decimal_add(d1: Decimal | None, d2: Decimal | None) -> Decimal:
    """Add two Decimal values.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        New Decimal containing the sum (d1 + d2)

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="10.5")
        >>> d2 = Decimal(value="20.3")
        >>> result = decimal_add(d1, d2)
        >>> result.value
        '30.8'
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return Decimal(value=str(val1 + val2))


def decimal_sub(d1: Decimal | None, d2: Decimal | None) -> Decimal:
    """Subtract two Decimal values.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (subtrahend, None treated as 0)

    Returns:
        New Decimal containing the difference (d1 - d2)

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="100.5")
        >>> d2 = Decimal(value="30.2")
        >>> result = decimal_sub(d1, d2)
        >>> result.value
        '70.3'
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return Decimal(value=str(val1 - val2))


def decimal_mul(d1: Decimal | None, d2: Decimal | None) -> Decimal:
    """Multiply two Decimal values.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        New Decimal containing the product (d1 * d2)

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="10.5")
        >>> d2 = Decimal(value="2")
        >>> result = decimal_mul(d1, d2)
        >>> result.value
        '21.0'
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return Decimal(value=str(val1 * val2))


def decimal_div(d1: Decimal | None, d2: Decimal | None) -> Decimal:
    """Divide two Decimal values.

    Args:
        d1: Dividend (None treated as 0)
        d2: Divisor (must not be zero, None treated as 0)

    Returns:
        New Decimal containing the quotient (d1 / d2)

    Raises:
        ZeroDivisionError: If d2 is zero

    None Safety:
        None inputs are treated as zero (note: None d2 will raise ZeroDivisionError)

    Example:
        >>> d1 = Decimal(value="100")
        >>> d2 = Decimal(value="4")
        >>> result = decimal_div(d1, d2)
        >>> result.value
        '25'
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)

    if val2 == 0:
        raise ZeroDivisionError("cannot divide by zero")

    return Decimal(value=str(val1 / val2))


def decimal_equal(d1: Decimal | None, d2: Decimal | None) -> bool:
    """Check if two Decimal values are equal.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        True if d1 == d2, False otherwise

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="10.5")
        >>> d2 = Decimal(value="10.5")
        >>> decimal_equal(d1, d2)
        True
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return val1 == val2


def decimal_less_than(d1: Decimal | None, d2: Decimal | None) -> bool:
    """Check if first Decimal is less than second.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        True if d1 < d2, False otherwise

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="10.5")
        >>> d2 = Decimal(value="20.3")
        >>> decimal_less_than(d1, d2)
        True
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return val1 < val2


def decimal_less_than_or_equal(d1: Decimal | None, d2: Decimal | None) -> bool:
    """Check if first Decimal is less than or equal to second.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        True if d1 <= d2, False otherwise

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="10.5")
        >>> d2 = Decimal(value="10.5")
        >>> decimal_less_than_or_equal(d1, d2)
        True
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return val1 <= val2


def decimal_greater_than(d1: Decimal | None, d2: Decimal | None) -> bool:
    """Check if first Decimal is greater than second.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        True if d1 > d2, False otherwise

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="20.3")
        >>> d2 = Decimal(value="10.5")
        >>> decimal_greater_than(d1, d2)
        True
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return val1 > val2


def decimal_greater_than_or_equal(d1: Decimal | None, d2: Decimal | None) -> bool:
    """Check if first Decimal is greater than or equal to second.

    Args:
        d1: First decimal value (None treated as 0)
        d2: Second decimal value (None treated as 0)

    Returns:
        True if d1 >= d2, False otherwise

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d1 = Decimal(value="10.5")
        >>> d2 = Decimal(value="10.5")
        >>> decimal_greater_than_or_equal(d1, d2)
        True
    """
    val1 = decimal_to_built_in(d1)
    val2 = decimal_to_built_in(d2)
    return val1 >= val2


def decimal_is_zero(d: Decimal | None) -> bool:
    """Check if Decimal value is zero.

    Args:
        d: Decimal value to check (None treated as 0)

    Returns:
        True if d == 0, False otherwise

    None Safety:
        None inputs are treated as zero (returns True)

    Example:
        >>> d = Decimal(value="0")
        >>> decimal_is_zero(d)
        True
        >>> d2 = Decimal(value="0.001")
        >>> decimal_is_zero(d2)
        False
    """
    val = decimal_to_built_in(d)
    return val == 0


def decimal_is_negative(d: Decimal | None) -> bool:
    """Check if Decimal value is negative (< 0).

    Args:
        d: Decimal value to check (None treated as 0)

    Returns:
        True if d < 0, False if d >= 0

    None Safety:
        None inputs are treated as zero (returns False)

    Example:
        >>> d = Decimal(value="-10.5")
        >>> decimal_is_negative(d)
        True
        >>> d2 = Decimal(value="10.5")
        >>> decimal_is_negative(d2)
        False
    """
    val = decimal_to_built_in(d)
    return val < 0


def decimal_is_positive(d: Decimal | None) -> bool:
    """Check if Decimal value is positive (> 0).

    Args:
        d: Decimal value to check (None treated as 0)

    Returns:
        True if d > 0, False if d <= 0

    None Safety:
        None inputs are treated as zero (returns False)

    Example:
        >>> d = Decimal(value="10.5")
        >>> decimal_is_positive(d)
        True
        >>> d2 = Decimal(value="0")
        >>> decimal_is_positive(d2)
        False
    """
    val = decimal_to_built_in(d)
    return val > 0


def decimal_round(d: Decimal | None, places: int) -> Decimal:
    """Round Decimal to specified number of decimal places.

    If places < 0, it rounds the integer part to the nearest 10^(-places).

    Args:
        d: Decimal value to round (None treated as 0)
        places: Number of decimal places (can be negative)

    Returns:
        New Decimal containing the rounded value

    None Safety:
        None inputs are treated as zero

    Example:
        >>> d = Decimal(value="5.45")
        >>> result = decimal_round(d, 1)
        >>> result.value
        '5.5'
        >>> d2 = Decimal(value="545")
        >>> result2 = decimal_round(d2, -1)
        >>> result2.value
        '550'
    """
    from decimal import Decimal as PyDecimal

    val = decimal_to_built_in(d)

    if places < 0:
        # For negative places, use shift-quantize-shift back approach
        # to properly round the integer part
        scale = PyDecimal(10) ** (-places)
        shifted = val / scale
        rounded_shifted = shifted.quantize(PyDecimal("1"), rounding=ROUND_HALF_UP)
        rounded_val = rounded_shifted * scale
    else:
        # For positive places, quantize directly
        quantizer = PyDecimal(10) ** -places
        rounded_val = val.quantize(quantizer, rounding=ROUND_HALF_UP)

    return Decimal(value=str(rounded_val))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meshtrade/testing/ledger/token_tap/v1/option.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'meshtrade/testing/ledger/token_tap/v1/option.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2meshtrade/testing/ledger/token_tap/v1/option.proto\x12%meshtrade.testing.ledger.token_tap.v1\"\x97\x01\n\x10MintTokenOptions\x12m\n\x14stellar_mint_options\x18\x01 \x01(\x0b\x32\x39.meshtrade.testing.ledger.token_tap.v1.StellarMintOptionsH\x00R\x12stellarMintOptionsB\x14\n\x12mint_token_options\"h\n\x12StellarMintOptions\x12R\n\x07options\x18\x01 \x03(\x0b\x32\x38.meshtrade.testing.ledger.token_tap.v1.StellarMintOptionR\x07options\"\xae\x01\n\x11StellarMintOption\x12\x81\x01\n\x1cstellar_mint_token_with_memo\x18\x01 \x01(\x0b\x32?.meshtrade.testing.ledger.token_tap.v1.StellarMintTokenWithMemoH\x00R\x18stellarMintTokenWithMemoB\x15\n\x13stellar_mint_option\".\n\x18StellarMintTokenWithMemo\x12\x12\n\x04memo\x18\x01 \x01(\tR\x04memoBt\n,co.meshtrade.api.testing.ledger.token_tap.v1ZDgithub.com/meshtrade/api/go/testing/ledger/token_tap/v1;token_tap_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtrade.testing.ledger.token_tap.v1.option_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n,co.meshtrade.api.testing.ledger.token_tap.v1ZDgithub.com/meshtrade/api/go/testing/ledger/token_tap/v1;token_tap_v1'
  _globals['_MINTTOKENOPTIONS']._serialized_start=94
  _globals['_MINTTOKENOPTIONS']._serialized_end=245
  _globals['_STELLARMINTOPTIONS']._serialized_start=247
  _globals['_STELLARMINTOPTIONS']._serialized_end=351
  _globals['_STELLARMINTOPTION']._serialized_start=354
  _globals['_STELLARMINTOPTION']._serialized_end=528
  _globals['_STELLARMINTTOKENWITHMEMO']._serialized_start=530
  _globals['_STELLARMINTTOKENWITHMEMO']._serialized_end=576
# @@protoc_insertion_point(module_scope)
